function [w,status] = hingereg(X,y)
% Hinge Regression: minimize the hinge loss on a sample of points.
%      w = hingereg(X,y)
% X contains the data vectors in its rows
% y is a column vector of -1/+1 labels.
% Returns:
% w a column vector, minimizing:
%   sum_i hingeloss(X(i,:)*w;y(i)
% where:
%   hingeloss(z,y) = max(0,1-y*z)
%
% Method: Uses MATLAB's built-in linear programming solver.
% (Actually, uses two different LP solvers, initializing also to the
% least-squares solution)


  [n,d] = size(X);

  % We set up the problem as an LP:
  %      Minimize sum err
  %      s.t. A w + I err >= 1 and err >=0
  % Putting this in canonical form:
  %      Minmize zeros*w + ones*err
  %      s.t. -A w + -I err <= 1
  %           w >= -inf   err >= 0
  %           w <= inf    err <= inf

  % The following should do it:
  tic;
  [errandwL,valL,exitflagL] = linprog(...
      [zeros(d,1);ones(n,1)],... %objective
      -[repmat(y,1,d).*X, speye(n)],-ones(n,1),... %inequality constraint
      [],[],... %no equality constraint
      [-inf(d,1);zeros(n,1)],... % lower bounds
      inf(d+n,1),...  % no upper bounds
      [],... %initialize search
      optimset('MaxIter',1000,'TolFun',1e-10,'Display','off'));
  status.exitflagL = exitflagL;
  w = errandwL(1:d);
  err = mean(max(1-y.*(X*w),0));
  status.errL = err;
  status.tocL = toc;
  
  % MATLAB's large scale LP methods sometimes don't work very well.  We
  % also try solving the least squares problem (corresponding to
  % loss(z,y)=(z-y)^2), and MATLAB's "medium scale" LP solver initialized
  % to the least squares solution.
  w0 = X\y;              % w0 = argmin_w |X*w-y|
  slack0 = max(1-y.*(X*w0),0);
  err0 = mean(slack0);
  status.err0 = err0;
  if err0<err,
    % least-squares is better then what large-scale found
    w = w0;
    err = err0;
  end
  % Now use the least-squares solution w0 to initialize a med-scale LP
  % solver:
  tic;
  [errandwM,valM,exitflagM] = linprog(...
      [zeros(d,1);ones(n,1)],... %objective
      -[repmat(y,1,d).*X, speye(n)],-ones(n,1),... %inequality constraint
      [],[],... %no equality constraint
      [-inf(d,1);zeros(n,1)],... % lower bounds
      inf(d+n,1),...  % no upper bounds
      [w0;slack0],... %initialize search
      optimset('LargeScale','off','MaxIter',1000,'TolFun',1e-8, ...
               'Display','off'));
  wM = errandwM(1:d);
  errM = mean(max(1-y.*(X*wM),0));
  status.exitflagM = exitflagM;
  status.errM = errM;
  status.tocM = toc;
  if errM<err,
    w = wM;
    err = errM;
  end
  