function [w,errs,runlog,w_deltas] = logisticreg(x,y,w0)
% Logistic Regression: minimize the logistic loss on a sample of points.
%      w = logisticreg(X,y)
% x contains the data vectors in its rows
% y is a column vector of -1/+1 labels.
% Returns:
% w a column vector, minimizing:
%   sum_i logloss(X(i,:)*w;y(i)
% where:
%   logloss(z,y) = log(1+exp(-y*z))
%
% Performs Conjugate Gradient Descent on the objective function.
% Initializes to the supplied w0, or the zero vector is w0=0 is provided,
% or to the least-squares solution is no w0 is provided.
%
% Based on code written by Thomas P Minka.
% Modified by Nati Srebro (removed regularization; changed stopping
% condition; introduced explicit reset to conjugate calculation;
% introduced armijo search for step size; changed input/output format;
% removed dependence on lightspeed).

[n,d] = size(x);
if nargin<3,
  w = x\y;
elseif length(w0)==1 && w0==0,
  w = zeros(d,1);
else
  w = w0;
end
x = (repmat(y,1,d).*x)'; % This is how Minka's code expects it

maxiter = 1e6;
armijo_sigma = 0.1;
armijo_beta = 0.5;

w_look_back_window = 100;
err_look_back_window = 1000;

old_g = zeros(size(w));
old_val = ll(w'*x);
for iter = 1:maxiter
  old_w = w;
  % s1 = 1-sigma
  s1 = 1./(1+exp(w'*x));
  g = x*s1';
  if iter == 1
    u = g;
  else
    u = cg_dir(u, g, old_g);
  end
  
  % newton step size along u
  ug = u'*g;
  ux = u'*x;
  a = s1.*(1-s1);
  uhu = (ux.^2)*a';
  stepsize = ug/uhu;
  % armijo line search
  while 1,
    new_val = ll((w+stepsize*u)'*x);
    if new_val < old_val - armijo_sigma * stepsize*ug,
      break;
    end
    stepsize=stepsize*armijo_beta;
    if stepsize<1e-30,
      runlog{iter} = ['armijo stuck with ug=' num2str(ug)];
      break;
    end
  end
  w = w + stepsize*u;
  old_g = g;
  
  errs(iter) = new_val/n; 
  old_val = new_val;
  w_deltas(iter) = max(abs(w-old_w));
  old_w = w;
  
  % 1st stopping condition: already seprable    
  margins = w'*x;
  if all(margins>0.1),
    w = 1e10*w;
    runlog{iter+1} = 'seprable';
    break % separable
  end
  
  % 2nd stopping condition: w hasn't changed for a while
  if iter>w_look_back_window && ...
        mean(w_deltas((iter-w_look_back_window):iter))<1e-8,
    runlog{iter+1} = 'no change in w';
    break % converged---  no change in w
  end
  
  % 3rd stopping condition: err isn't decreasing
  if iter>err_look_back_window && ...
        errs(iter)>errs(iter-err_look_back_window)-1e-4,
    runlog{iter+1} = 'no reduction in err';
    break % converged--- no change in objective
  end
end
if iter == maxiter
  warning('not enough iters')
end


function dir = cg_dir(old_dir, grad, old_grad)
% Compute the new conjugate direction.

g = grad;
grad = grad(:);
old_grad = old_grad(:);

% Hestenes-Stiefel
delta = grad - old_grad;
beta = (grad'*delta) / (old_dir'*delta);
% Polak-Ribiere
%beta = -grad'*(grad - old_grad) / (old_grad'*old_grad);
% Fletcher-Reeves
%beta = -(grad'*grad) / (old_grad'*old_grad);

dir = g - beta*old_dir;
if dir'*grad < 0.01*grad'*grad,
  dir = grad;
end

function e = ll(margins)
  e = sum(max(0,-margins)+log(1+exp(-abs(margins))));

