function h = showdata(x,y,trainx,trainy,pred,varargin)
% Shows a distribution, and optionally a training set and a decision
% boundary
%   showdata(x,y)
%   showdata(x,y,trainx,trainy)
%   showdata(x,y,trainx,trainy, @funcname, ...)
%     e.g.:
%       showdata(x,y,trainx,trainy, @mypredictor, 4, w)
%     where mypredictor(x,4,w) returns predictions
%   showdata(x,y,trainx,trainy, @(x) <expression>)
%     e.g.:
%       showdata(x,y,trainx,trainy, @(x) sign(x*w))
%       showdata(x,y,trainx,trainy, @(x) mypredictor(x,4,w))
% Inputs:
% x - an n*2 matrix with one data point per row
% y - a vector of n -1/+1 labels
%   x,y represent the "true" distribution and are plotted as dots.
% trainx - an m*2 matrix with one data point per row
% trainy - a vector of m -1/+1 labels
%   if present, these are plotted with larger markers
% funcname - a function that is called with a matrix of points (one per
%   row) as its first argument, and any additional arguments passed
%   along.  It should return a vector of predictions (only the sign of
%   which is important).
% OR:
% A prediction function can be specified using the @(x) <expression>
% notation.
  
  
  h = figure;
  hold;

  % Plot the distribution
  plot(x(y<0,1),x(y<0,2),'r.',...
       x(y>0,1),x(y>0,2),'b.','MarkerSize',2);
  
  if nargin>2, % training set supplied
    % Plot the training set
    plot(trainx(trainy<0,1),trainx(trainy<0,2),'r*',...
         trainx(trainy>0,1),trainx(trainy>0,2),'b*','MarkerSize',5);
  end
  
  if nargin>4,  % prediction function supplied
    [x1,x2] = meshgrid(0:0.01:1,0:0.01:1);
    [s1 s2] = size(x1);
    yy = sign(feval(pred,[reshape(x1,[],1) reshape(x2,[],1)],...
                    varargin{:}));
    yy = reshape(yy,s1,s2);
    % Plot the decision boundary
    contour(x1,x2,yy,[0 0],'k');
  end

